<?php

// main  Simple 3D Coverflow plugin class
class FWDS3DCov
{
	// constants
	const MIN_WP_VER =  "3.5.0";
	const CAPABILITY = "edit_fwds3dcov";
	const VERSION = "1.0";
	
	// variables
	private $_data;
	private $_dir_url;
    
    private static $_cov_id = 0;
    
    // constructor
    public function init()
    {
		$this->_dir_url = plugin_dir_url(dirname(__FILE__));
		
		// set data
		$this->_data = new FWDS3DCovData();		
		$this->_data->init();
		
    	// set hooks
    	add_action("admin_menu", array($this, "add_plugin_menu"));
		add_action("wp_enqueue_scripts", array($this, "add_scripts_and_styles"));
		add_shortcode("fwds3dcov", array($this, "set_coverflow"));
    }

    // functions
    public function add_plugin_menu(){
    	// add menus
        add_menu_page("Coverflow Title", " Simple 3D Coverflow", FWDS3DCov::CAPABILITY, "FWDS3DCovMenu-General-Settings", array($this, "set_general_settings"), $this->_dir_url . "load/icons/menu-icon.png");
       	add_submenu_page("FWDS3DCovMenu-General-Settings", "General settings", "General settings", FWDS3DCov::CAPABILITY, "FWDS3DCovMenu-General-Settings");
       	add_submenu_page("FWDS3DCovMenu-General-Settings", "Playlists manager", "Playlists manager", FWDS3DCov::CAPABILITY, "FWDS3DCovMenu-Playlists-Manager", array($this, "set_playlists_manager"));
       	add_submenu_page("FWDS3DCovMenu-General-Settings", "CSS Editor", "CSS Editor", FWDS3DCov::CAPABILITY, "FWDS3DCovMenu-CSS-Editor", array($this, "set_css_editor"));
       	
      	// add shortcode metabox
       	$post_type_screens = array("post", "page");
       	$args = array(
	       'public'   => true,
	       '_builtin' => false,
	    );
	    
       	$custom_post_types = get_post_types($args);
       	foreach ($custom_post_types as $screen){
       		$post_type_screens[] = $screen;
       	}

    	foreach ($post_type_screens as $screen){
       		add_meta_box("fwds3dcov-shortcode-generator", "Simple 3D Coverflow Shortcode Generator",  array($this, "set_custom_meta_box"), $screen, "side", "default");
    	}
    }
    
	private function check_wp_ver(){
	    global $wp_version;
	    
		$exit_msg = "The  Simple 3D Coverflow plugin requires WordPress " . FWDS3DCov::MIN_WP_VER . " or newer. <a href='http://codex.wordpress.org/Updating_WordPress'>Please update!</a>";
		
		if (version_compare($wp_version, FWDS3DCov::MIN_WP_VER) <= 0){
			echo $exit_msg;
			
			return false;
		}
		
		return true;
	}

    public function set_general_settings(){
    	if (!$this->check_wp_ver()){
    		return;
    	}
    	
    	$msg = "";
    	
    	$set_id = 0;
		$set_order_id = 0;
		$tab_init_id = 0;
    	
	    if (!empty($_POST) && check_admin_referer("fwds3dcov_general_settings_update", "fwds3dcov_general_settings_nonce")){
			$data_obj = json_decode(str_replace("\\", "", $_POST["settings_data"]), true);
			
			$action = $data_obj["action"];
			$settingsAr = $data_obj["settings_ar"];

			//validate input
			foreach ($settingsAr as $key => $value) {
					
				if(!empty($settingsAr[$key]["box_shadow_css"])){
					$settingsAr[$key]["box_shadow_css"] = sanitize_text_field($settingsAr[$key]["box_shadow_css"]);
				}
				if(!empty($settingsAr[$key]["skin_path"])){
					$settingsAr[$key]["skin_path"] = sanitize_text_field($settingsAr[$key]["skin_path"]);
				}

				if(!empty($settingsAr[$key]["cov_bg_image_url"])){
					$settingsAr[$key]["cov_bg_image_url"] = esc_url($settingsAr[$key]["cov_bg_image_url"]);
				}
			}

			$this->_data->settings_ar = $settingsAr;
			$this->_data->set_data();
			
			switch ($action){
			    case "add":
			        $msg = "<div class='fwd-updated'><p style='padding:8px;'>Your new preset has been added!</p></div>";
			        $set_id = $data_obj["set_id"];
					$set_order_id = $data_obj["set_order_id"];
					$tab_init_id = $data_obj["tab_init_id"];
			        break;
			    case "save":
			        $msg = "<div class='fwd-updated'><p style='padding:8px;'>Your preset settings have been updated!</p></div>";
			        $set_id = $data_obj["set_id"];
					$set_order_id = $data_obj["set_order_id"];
					$tab_init_id = $data_obj["tab_init_id"];
			        break;
			    case "del":
			       	$msg = "<div class='fwd-updated'><p style='padding:8px;'>Your preset has been deleted!</p></div>";
			        break;
			}
		}
		
		// jquery ui
		wp_enqueue_style("fwds3dcov_fwd_ui_css", $this->_dir_url . "css/fwd_ui.css", array(), '1.10.4');
		wp_enqueue_script("fwds3dcov_fwdtooltip", $this->_dir_url . "js/fwdtooltip.js", array(), '1.0');
		wp_enqueue_script("jquery-ui-tabs");
		wp_enqueue_script("jquery-ui-sortable");
		wp_enqueue_script("jquery-ui-accordion");
		
		// spectrum colorpicker
    	wp_enqueue_style("fwds3dcov_spectrum_css", $this->_dir_url . "css/spectrum.css");
    	wp_enqueue_script("fwds3dcov_spectrum_script", $this->_dir_url . "js/spectrum.js");
    	
    	// general settings script
		wp_enqueue_media();
        wp_enqueue_script("fwds3dcov_general_settings_script", $this->_dir_url . "js/general_settings.js");
		
    	include_once "general_settings.php";
    }
    
 	public function set_playlists_manager(){
    	if (!$this->check_wp_ver()){
    		return;
    	}
    	
    	$msg = "";
    	
	    if (!empty($_POST) && check_admin_referer("fwds3dcov_playlist_manager_update", "fwds3dcov_playlist_manager_nonce")){
			$playlistsAr = json_decode(str_replace("\\", "", $_POST["playlist_data"]), true);
			
			$this->_data->playlists_ar = $playlistsAr;
			$this->_data->set_data();
			
			$msg = "<div class='fwd-updated'><p style='padding:8px;'>Your playlists have been updated!</p></div>";
		}
		
		// jquery ui
		wp_enqueue_style("fwds3dcov_fwd_ui_css", $this->_dir_url . "css/fwd_ui.css", array(), '1.10.4');
		wp_enqueue_script("fwds3dcov_fwdtooltip", $this->_dir_url . "js/fwdtooltip.js", array(), '1.0');
		wp_enqueue_script("jquery-ui-tabs");
		wp_enqueue_script("jquery-ui-sortable");
		wp_enqueue_script("jquery-ui-accordion");
		wp_enqueue_script("jquery-ui-dialog");
		
		// playlist manager script
		wp_enqueue_media();
        wp_enqueue_script("fwds3dcov_playlist_manager_script", $this->_dir_url . "js/playlist_manager.js");
        
    	include_once "playlist_manager.php";
    }
    
    public function set_css_editor(){
    	if (!$this->check_wp_ver()){
    		return;
    	}
    	
    	$msg = "";
    	$scroll_pos = 0;
    	
    	$css_file = plugin_dir_path(dirname(__FILE__)) . "css/fwds3dcov.css";
    	
	    if (!empty($_POST) && check_admin_referer("fwds3dcov_css_editor_update", "fwds3dcov_css_editor_nonce")){
			$handle = fopen($css_file, "w") or die("Cannot open file: " . $css_file);
			
			$data = str_replace("\\", "", $_POST["css_data"]);
			$data = str_replace("'e", "'\\e", $data);
			$scroll_pos = $_POST["scroll_pos"];
			
			fwrite($handle, $data);
			
			$msg = "<div class='fwd-updated'><p style='padding:8px;'>The CSS file has been updated!</p></div>";
		}
		
		wp_enqueue_style("fwds3dcov_fwd_ui_css", $this->_dir_url . "css/fwd_ui.css");
	  			
		$handle = fopen($css_file, "r") or die("Cannot open file: " . $css_file);
        
    	include_once "css_editor.php";
    	
    	fclose($handle);
    }

	public static function set_action_links($links){
		$settings_link = "<a href='" . get_admin_url(null, "admin.php?page=FWDS3DCovMenu-General-Settings") . "'>Settings</a>";
   		array_unshift($links, $settings_link);
   		
   		return $links;
	}

	public function add_scripts_and_styles(){
     	global $post, $wpdb;
     	if(empty($post)) return;

		$shortcode_found = false;
       	if(has_shortcode($post->post_content, 'fwds3dcov')){
          	$shortcode_found = true;
       	}else if(isset($post->ID)){ 
          	$result = $wpdb->get_var($wpdb->prepare(
            "SELECT count(*) FROM $wpdb->postmeta " .
            "WHERE post_id = %d and meta_value LIKE '%%fwds3dcov%%'", $post->ID));
          	$shortcode_found = !empty($result);
       	}
       
		if (!empty($shortcode_found)){
			wp_enqueue_style("fwds3dcov_cov_css", $this->_dir_url . "css/fwds3dcov.css");
    		wp_enqueue_script("fwds3dcov_cov_script", $this->_dir_url . "js/FWDSimple3DCoverflow.js");
		}		
	}
    
    public function get_constructor($sid, $pid){
    	$preset = NULL;
    	
    	foreach ($this->_data->settings_ar as $set){
    		if ($set["name"] == $sid){
    			$preset = $set;
    		}
    	}
    	
    	if (is_null($preset)){
    		return "Preset with id ". $sid . " does not exist!";
    	}
    	
    	$playlist = NULL;
    	
    	foreach ($this->_data->playlists_ar as $pl){
    		if ($pl["name"] == $pid){
    			$playlist = $pl;
    		}
    	}
    	 	
    	if (is_null($playlist)){
    		return "Playlist with id ". $pid . " does not exist!";
    	}

    	
    	return "<script type='text/javascript'>document.addEventListener('DOMContentLoaded', function(event) {if(!FWDRLS3DUtils.checked) FWDRLS3DUtils.checkIfHasTransofrms();FWDRLS3DUtils.checked = true;window.fwds3dcov" . FWDS3DCov::$_cov_id. " = new FWDSimple3DCoverflow({" . "coverflowHolderDivId:'fwds3dcovDiv" . FWDS3DCov::$_cov_id. "',coverflowDataListDivId:'fwds3dcovPlaylist" . $pid . "', mainFolderPath:'" . $this->_dir_url . "load'," ."displayType:'" . $preset['display_type'] . "',coverflowWidth:" . $preset['cov_width'] . ",coverflowOffsetHeight:" . $preset['coverflowOffsetHeight'] . ",skinPath:'" . $preset['skin_path'] . "',backgroundColor:'" . $preset['cov_bg_color'] . "',descriptionGradientWindowBackgroundColor1:'" . $preset['descriptionGradientWindowBackgroundColor1'] .  "',descriptionGradientWindowBackgroundColor2:'" . $preset['descriptionGradientWindowBackgroundColor2'] . "',slideshowPreloaderBackgroundColor:'" . $preset['slideshowPreloaderBackgroundColor'] .  "',slideshowpreloaderFillColor:'" . $preset['slideshowpreloaderFillColor'] . "',preloaderBackgroundColor:'" . $preset['preloaderBackgroundColor'] . "',preloaderFillColor:'" . $preset['preloaderFillColor'] . "',backgroundImagePath:'" . $preset['cov_bg_image_url'] . "',backgroundPosition:'" . $preset['backgroundPosition'] . "',backgroundSize:'" . $preset['backgroundSize'] . "',backgroundRepeat:'" . $preset['bg_repeat'] . "',coverflowStartPosition:'" . $preset['cov_start_pos'] . "',coverflowTopology:'" . $preset['cov_topology'] . "',coverflowXRotation:" . $preset['cov_x_rot'] . ",coverflowYRotation:" . $preset['cov_y_rot'] . ",numberOfThumbnailsToDisplayLeftAndRight:" . $preset['nr_side_thumbs'] . ",infiniteLoop:'" . $preset['infinite_loop'] . "',autoplay:'" . $preset['cov_autoplay'] . "',slideshowDelay:" . $preset['cov_slideshow_delay'] . ",rightClickContextMenu:'" . $preset['right_click_context_menu'] . "',addKeyboardSupport:'" . $preset['cov_keyboard_support'] . "', useDragAndSwipe:'" . $preset['use_drag_and_swipe'] . "', fluidWidthZIndex:" . $preset['fluid_width_z_index'] . "," . "thumbnailXOffset3D:" . $preset['thumb_x_offset_3d'] . ",useDefaultThumbnailsWidthAndHeight:'" . $preset['useDefaultThumbnailsWidthAndHeight'] . "',thumbnailWidth:" . $preset['thumbnailWidth'] . ",thumbnailHeight:" . $preset['thumbnailHeight']  . ",thumbnailMaxHeight:" . $preset['thumbnailMaxHeight'] . ",thumbnailXSpace3D:" . $preset['thumb_x_space_3d'] . ",thumbnailZOffset3D:" . $preset['thumb_z_offset_3d'] . ",thumbnailZSpace3D:" . $preset['thumb_z_space_3d'] . ",thumbnailYAngle3D:" . $preset['thumb_y_angle_3d'] . ",thumbnailHoverOffset:" . $preset['thumb_hover_offset'] . ",thumbnailBorderSize:" . $preset['thumb_border_size'] . ",thumbnailOffsetY:" . $preset['thumbnailOffsetY'] . ",thumbnailBackgroundColor:'" . $preset['thumb_bg_color'] . "',thumbnailBorderColor1:'" . $preset['thumb_border_color1'] . "',thumbnailBorderColor2:'" . $preset['thumb_border_color2'] . "',transparentImages:'" . $preset['transparent_images'] . "',thumbnailsAlignment:'" . $preset['thumbs_alignment'] . "',maxNumberOfThumbnailsOnMobile:" . $preset['max_thumbs_mobile'] . ",showThumbnailsGradient:'" . $preset['show_thumbs_gradient'] . "',thumbnailGradientColor1:'" . $preset['thumb_gradient_color1'] . "',thumbnailGradientColor2:'" . $preset['thumb_gradient_color2'] . "',showText:'" . $preset['show_thumbs_text'] . "',textMaxWidth:" . $preset['textMaxWidth'] . ",textOffset:" . $preset['text_offset'] . ",showThumbnailBoxShadow:'" . $preset['show_thumb_box_shadow'] . "',thumbnailBoxShadowCss:'" . $preset['box_shadow_css'] . "',showReflection:'" . $preset['show_reflection'] . "',reflectionHeight:" . $preset['reflection_height'] . ",reflectionDistance:" . $preset['reflection_distance'] . ",reflectionOpacity:" . $preset['reflection_opacity'] . "," . "controlsMaxWidth:" . $preset['controls_max_width'] . ",controlsPosition:'" . $preset['controls_position'] . "',controlsOffset:'" . $preset['controls_offset'] . "',showNextAndPrevCoverflowButtons:'" . $preset['show_next_and_prev_coverflow_buttons'] . "',disableNextAndPrevButtonsOnMobile:'" . $preset['disable_btns_mobile'] . "',showSlideshowButton:'" . $preset['show_slideshow_btn'] . "',showScrollbar:'" . $preset['show_scrollbar'] . "',scrollbarHandlerWidth:" . $preset['scrollbar_handler_width'] . ",scrollbarTextColorNormal:'" . $preset['scrollbar_text_normal_color'] . "',scrollbarTextColorSelected:'" . $preset['scrollbar_text_selected_color'] . "',disableScrollbarOnMobile:'" . $preset['disable_scrollbar_mobile'] . "',showLargeNextAndPrevCoverflowButtons:'" . $preset['show_large_next_and_prev_coverflow_buttons'] . "',largeNextAndPrevButtonsOffest:" . $preset['large_next_and_prev_buttons_offest'] . ",nextAndPrevCoverflowButtonsOffsetX:" . $preset['nextAndPrevCoverflowButtonsOffsetX'] . ",largeNextAndPrevButtonsMaxWidthPos:" . $preset['largeNextAndPrevButtonsMaxWidthPos'] . ",showBulletsNavigation:'" . $preset['show_bullets_navigation'] . "',bulletsNormalColor:'" . $preset['bullets_normal_color'] . "',bulletsSelectedColor:'" . $preset['bullets_selected_color'] . "',bulletsNormalRadius:" . $preset['bullets_normal_radius'] . ",bulletsSelectedRadius:" . $preset['bullets_selected_radius'] . ",spaceBetweenBullets:" . $preset['space_between_bullets'] . ",bulletsOffset:" . $preset['bullets_offset'] . ",enableMouseWheelScroll:'" . $preset['enable_mouse_wheel'] . "'," . "showCategoriesMenu:'" . $preset['show_categories_menu'] . "',startAtCategory:" . $preset['start_category'] . ",categoriesMenuMaxWidth:" . $preset['categories_menu_max_width'] . ",categoriesMenuOffset:" . $preset['categories_menu_offset'] . ",categoryColorNormal:'" . $preset['category_normal_color'] . "',categoryColorSelected:'" . $preset['category_selected_color'] . "'," . "buttonsAlignment:'" . $preset['buttons_alignment'] . "',itemBoxShadow:'" . $preset['item_box_shadow'] . "',descriptionWindowAnimationType:'" . $preset['description_window_animation_type'] . "',descriptionWindowPosition:'" . $preset['description_window_position'] . "',slideShowAutoPlay:'" . $preset['slide_show_auto_play'] . "',addKeyboardSupport:'" . $preset['add_keyboard_support'] . "',showCloseButton:'" . $preset['show_close_button'] . "',showZoomButton:'" . $preset['show_zoom_button'] . "',showShareButton:'" . $preset['show_share_button'] . "',showSlideShowButton:'" . $preset['show_slide_show_button'] . "',showSlideShowAnimation:'" . $preset['show_slide_show_animation'] . "',showNextAndPrevButtons:'" . $preset['show_next_and_prev_buttons'] . "',showNextAndPrevButtonsOnMobile:'" . $preset['show_next_and_prev_buttons_on_mobile'] . "',showDescriptionButton:'" . $preset['show_description_button'] . "',showDescriptionByDefault:'" . $preset['show_description_by_default'] . "',videoShowFullScreenButton:'" . $preset['video_show_full_screen_button'] . "',videoAutoPlay:'" . $preset['video_auto_play'] . "',nextVideoOrAudioAutoPlay:'" . $preset['next_video_or_audio_auto_play'] . "',videoLoop:'" . $preset['video_loop'] . "',audioAutoPlay:'" . $preset['audio_auto_play'] . "',audioLoop:'" . $preset['audio_loop'] . "',backgroundOpacity:" . $preset['background_opacity'] . ",descriptionWindowBackgroundOpacity:" . $preset['description_window_background_opacity'] . ",buttonsHideDelay:" . $preset['buttons_hide_delay'] . ",slideShowDelay:" . $preset['slide_show_delay'] . ",defaultItemWidth:" . $preset['default_item_width'] . ",defaultItemHeight:" . $preset['default_item_height'] . ",itemOffsetHeight:" . $preset['item_offset_height'] . ",spaceBetweenButtons:" . $preset['space_between_buttons'] . ",buttonsOffsetIn:" . $preset['buttons_offset_in'] . ",buttonsOffsetOut:" . $preset['buttons_offset_out'] . ",itemBorderSize:" . $preset['item_border_size'] . ",itemBorderRadius:" . $preset['item_border_radius'] . ",itemBackgroundColor:'" . $preset['item_background_color'] . "',lightBoxbackgroundColor:'" . $preset['light_box_background_color'] . "',videoPosterBackgroundColor:'" . $preset['video_poster_background_color'] . "',videoControllerBackgroundColor:'" . $preset['video_controller_background_color'] . "',audioControllerBackgroundColor:'" . $preset['video_controller_background_color'] . "',timeColor:'" . $preset['time_color'] . "'})});</script>";
    }
    
    public function get_playlist($pid){
    	$playlist = NULL;
    	
    	foreach ($this->_data->playlists_ar as $pl){
    		if ($pl["name"] == $pid){
    			$playlist = $pl;
    		}
    	}
    	
    	if (is_null($playlist)){
    		return "Playlist with id ". $pid . " does not exist!";
    	}

    
    	if($playlist['plType'] && $playlist['plType'] == 'instagram'){
    		$playlist_str = "<ul id='fwds3dcovPlaylist$pid' data-source='" . $playlist['name'] . "' style='display: none;'>";
    		$playlist_str .= "</ul>";
    	}else{
    		$playlist_str = "<ul id='fwds3dcovPlaylist$pid' style='display: none;'>";
    	
	    	foreach ($playlist["categories"] as $category){
	    		$playlist_str .= "<ul data-cat='" . $category["name"] . "'>";
	    		
	    		foreach ($category["thumbs"] as $thumb){
					$playlist_str .= "<ul>";
					$url = $thumb["url"];
					//$url = str_replace("link:", "", $url);
					//$url = str_replace("link:", "", $url);
					//$url = str_replace("?target=_blank", "", $url);
					//$url = str_replace("?target=_self", "", $url);
					//$url = str_replace("target=_blank", "", $url);
					//$url = str_replace("target=_self", "", $url);
					$playlist_str .= "<li data-url='" . $url . "'";
					
					
					if(strpos($thumb["url"], "target=_self") !== false){
						$playlist_str .= " data-target='_self'";
					}else if(strpos($thumb["url"], "target=_blank") !== false){
						$playlist_str .= " data-target='_blank'";
					}
					
					
					if(isset($thumb["video_poster_source"])){
						$playlist_str .= " data-poster-path='" . $thumb["video_poster_source"] . "'";
					}
					
					if(isset($thumb["item_max_width"])){
						$playlist_str .= " data-width='" . $thumb["item_max_width"] . "'";
					}
					
					if(isset($thumb["item_max_height"])){
						$playlist_str .= " data-height='" . $thumb["item_max_height"] . "'";
					}
					
					$playlist_str .= "></li>";
					$playlist_str .= "<li data-thumbnail-path='" . $thumb["path"] . "' data-thumbnail-width='" .  $thumb["width"] . "' data-thumbnail-height='" .  $thumb["height"] . "'></li><li data-thumbnail-text=''>" . $thumb["text"] . "</li>";

					if(!empty($thumb["info"])){
						$playlist_str .= "<li data-info=''>" . $thumb["info"] . "</li>";
					}
						
					$playlist_str .= "</ul>";
	    		}
	    		
	    		$playlist_str .= "</ul>";
	    	}
	    	
	    	$playlist_str .= "</ul>";
    	}
    	
    	return $playlist_str;
    }
    
 	public function set_coverflow($atts){
		extract(shortcode_atts(array("preset_id" => 0, "playlist_id" => 0), $atts, "fwds3dcov"));
		
		$cov_constructor = $this->get_constructor($preset_id, $playlist_id);
		$cov_div = "<div id='fwds3dcovDiv" . FWDS3DCov::$_cov_id. "'></div>";
		$cov_playlist = $this->get_playlist($playlist_id);
		
		FWDS3DCov::$_cov_id++;
		
		$cov_output = $cov_constructor . $cov_div . $cov_playlist;
		
		return $cov_output;
	}
	
	public function set_custom_meta_box($post){
		if (!$this->check_wp_ver()){
    		return;
    	}
		
		// presets
		$presetsNames = array();
		
		foreach ($this->_data->settings_ar as $setting){
    		$el = array(
    						"id" => $setting["id"],
    						"name" => $setting["name"]
    				   );
    				   
    		array_push($presetsNames, $el);
    	}
    	
		// playlists
		$playlistsNames = array();
		
		if (isset($this->_data->playlists_ar)){
			foreach ($this->_data->playlists_ar as $playlist){
	    		$el = array(
	    						"id" => $playlist["id"],
	    						"name" => $playlist["name"]
	    				   );
	    				   
	    		array_push($playlistsNames, $el);
	    	}
		}
		
    	wp_enqueue_style("fwds3dcov_fwd_ui_css", $this->_dir_url . "css/fwd_ui.css");
		wp_enqueue_script("fwds3dcov_shortcode_script", $this->_dir_url . "js/shortcode.js");
		
    	include_once "meta_box.php";
	}
}

?>